//+------------------------------------------------------------------+ 
//|   Statistical Analysis using Bollinger Bands - for FX v1.0       |
//|                                                                  | 
//|   Copyright  2014 / airquest@hotmail.com                        |
//|                                                                  |
//|   http://smart-trading.world-record.ch                           |  
//+------------------------------------------------------------------+ 

#property copyright   "Copyright  2014 // airquest // smart-trading.world-record.ch"
#property link        "http://smart-trading.world-record.ch"
#property description "Statistical Analysis using Bollinger Bands - for FX"
#property version     "1.0"

// Explanations :
// - Calculates the Win rate and Balance for a reversal when the candle breaks the bands.
// - Entry on open of next candle and exit on TP or SL.
// - For entry, CCI has to be OB or OS on the same candle.
// - No trades : 1) When there is already a trade open; 2) When both High and Low have broken the bands.

//+------------------------------------------------------------------+
//| Buffers                                                          |
//+------------------------------------------------------------------+

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1 Gray
#property indicator_color2 Red
#property indicator_color3 Green
#property indicator_style1 STYLE_DOT
#property indicator_width2 2
#property indicator_width3 2

//+------------------------------------------------------------------+
//| Enumerations                                                     |
//+------------------------------------------------------------------+

enum ENUM_PRICES
{
   close          = 0,  // Close
   high_low       = 1,  // High-Low
   median         = 2,  // Median
   typical        = 3,  // Typical
   weighted_close = 4   // Weighted Close
};

//+------------------------------------------------------------------+

enum ENUM_COMMENTS
{
   labels    = 0,  // Labels
   comments  = 1   // Comments
};

//+------------------------------------------------------------------+

enum ENUM_OPTI
{
   win_rate = 0,  // Win Rate
   balance  = 1   // Balance
};

//+------------------------------------------------------------------+
//| External variables                                               |
//+------------------------------------------------------------------+

extern int               BandsPeriod   = 20;
input ENUM_MA_METHOD     BandsMethod   = MODE_SMA;
input ENUM_APPLIED_PRICE BandsPrice    = PRICE_CLOSE;
extern double            Deviation     = 2.0;
extern int               BandsShift    =  0;
input ENUM_PRICES        PriceForBreak = high_low;

extern int               CCIPeriod     = 14;
input ENUM_APPLIED_PRICE CCIPrice      = PRICE_TYPICAL;
extern int               CCIOBOSLevel  = 150;

extern string            _Optimization = "=============== Optimization ===============";
extern bool              Optimize      = False;
input ENUM_OPTI          OptimizeWhat  = win_rate;
extern double            StopLoss      = 300;
extern double            TakeProfit    = 500;
extern double            Spread        = 10;
extern double            Lots          = 1.0;

extern string            _BB           = "---------------------- Bollinger Bands ----------------------";
extern int               PeriodMin     = 5;
extern int               PeriodMax     = 100;
extern int               PeriodStep    = 5;
input ENUM_MA_METHOD     MethodMin     = MODE_SMA;
input ENUM_MA_METHOD     MethodMax     = MODE_SMA;
input ENUM_APPLIED_PRICE PriceMin      = PRICE_CLOSE;
input ENUM_APPLIED_PRICE PriceMax      = PRICE_CLOSE;
extern double            DeviationMin  = 2.0;
extern double            DeviationMax  = 2.0;
extern double            DeviationStep = 0.2;

extern string            _CCI          = "-------------- Commodity Channel Index --------------";
extern int               CCIPeriodMin  = 14;
extern int               CCIPeriodMax  = 14;
extern int               CCIPeriodStep =  1;

input ENUM_APPLIED_PRICE CCIPriceMin   = PRICE_TYPICAL;
input ENUM_APPLIED_PRICE CCIPriceMax   = PRICE_TYPICAL;

extern int               CCILevelMin   = 150;
extern int               CCILevelMax   = 150;
extern int               CCILevelStep  =  25;

extern string            _Others                  = "---------------------- Others Settings ----------------------";
extern bool              AllowMultipleOpenTrades  = True;
extern int               BarsToCount              = 1000;
extern bool              UseAllBars               = True;
extern bool              RunOnlyOnce              = True;
extern bool              DrawArrows               = True;
extern bool              DrawSL_TP                = True;
extern bool              ShowArrowsOnEntryCandle  = False;
extern bool              ShowSL_TPOnEntryCandle   = False;
extern bool              ShowComments             = True;
int                      CommentsMode             = 1;
extern bool              SoundAtTheEnd            = True;
extern string            Sound                    = "alert2.wav";

extern string            _CSVFile                 = "============= CSV File Writing==============";
extern bool              WriteCSVFile             = False;
extern bool              WriteOnlyOnOptimization  = True;
extern string            NameOfFileToWrite        = "StatAnal-1";
extern bool              AddSymbolToName          = True;
extern bool              AddTimeFrameToName       = True;
extern bool              AddIndicatorName         = True;
extern string            IndicatorName            = "BB+CCI_FX";

//+------------------------------------------------------------------+
//| Internal variables                                               |
//+------------------------------------------------------------------+

double MovingBuffer[];
double UpperBuffer[];
double LowerBuffer[];

double Loss[];
double Win[];
double Balance[];

double SL[];
double TP[];
double RecSL[];
double RecTP[];
double Direction[];
double FirstLossShift[];
double FirstWinShift[];

double PriceUp[];
double PriceDn[];

double Osc[];

int    BandPeriod;
int    BandPrice;
int    BandMethod;
double BandDeviation;
int    CCIperiod;
int    CCIprice;
int    CCILevel;
bool   file;
string priceForBreak;
string fileName;
string TFName,symbol,ID;
int    point;

//+------------------------------------------------------------------+
//| Initialization function                                          |
//+------------------------------------------------------------------+

int init()
{
   IndicatorDigits(Digits);
   IndicatorBuffers(16);

   SetIndexBuffer(0,MovingBuffer);  SetIndexLabel(0,"MA");
   SetIndexBuffer(1,UpperBuffer);   SetIndexLabel(1,"Upper Band");
   SetIndexBuffer(2,LowerBuffer);   SetIndexLabel(2,"Lower Band");

   SetIndexDrawBegin(0,BandsPeriod+BandsShift);    SetIndexShift(0,BandsShift);
   SetIndexDrawBegin(1,BandsPeriod+BandsShift);    SetIndexShift(1,BandsShift);
   SetIndexDrawBegin(2,BandsPeriod+BandsShift);    SetIndexShift(2,BandsShift);

   SetIndexBuffer(3,Loss);             SetIndexLabel(3,"Loss");
   SetIndexBuffer(4,Win);              SetIndexLabel(4,"Win");
   SetIndexBuffer(5,Balance);          SetIndexLabel(5,"Balance");
   SetIndexBuffer(6,SL);               SetIndexLabel(6,"Stop Loss");
   SetIndexBuffer(7,TP);               SetIndexLabel(7,"Take Profit");
   SetIndexBuffer(8,RecSL);            SetIndexLabel(8,"Rec SL");
   SetIndexBuffer(9,RecTP);            SetIndexLabel(9,"Rec TP");
   SetIndexBuffer(10,Direction);       SetIndexLabel(10,"Direction");
   SetIndexBuffer(11,FirstLossShift);  SetIndexLabel(11,"First Loss Shift");
   SetIndexBuffer(12,FirstWinShift);   SetIndexLabel(12,"First Win Shift");

   SetIndexBuffer(13,PriceUp);         SetIndexLabel(13,"Price High");
   SetIndexBuffer(14,PriceDn);         SetIndexLabel(14,"Price Low");  
   
   SetIndexBuffer(15,Osc);             SetIndexLabel(15,"Oscillator");

   for(int i=3; i<16; i++) SetIndexStyle(i,DRAW_NONE);

   switch(PriceForBreak)
   {
      case 0: priceForBreak = "Close";          break;
      case 1: priceForBreak = "High-Low";       break;
      case 2: priceForBreak = "Median";         break;
      case 3: priceForBreak = "Typical";        break;
      case 4: priceForBreak = "Weigthed Close"; break;
   }

//+------------------------------------------------------------------+ Init writing of CSV file

   if(WriteCSVFile)
   {
      if(AddTimeFrameToName) TFName = TimeFrameToString(Period())+"-"; else TFName = "";
      if(AddSymbolToName)    symbol = Symbol()+"-";                    else symbol = "";
      if(AddIndicatorName)   ID     = IndicatorName+"-";               else ID     = "";
      
      fileName = ID+symbol+TFName+NameOfFileToWrite+".csv";

      if(FileIsExist(fileName)) FileDelete(fileName);

      file = FileOpen(fileName,FILE_CSV|FILE_READ|FILE_WRITE,',',CP_UTF8);

      if(!file)
      {
         Alert("File "+fileName+" not found, the last error is ", GetLastError());
      }
      else
      {
         FileWrite(file,"Balance","Win%","Total Trades","Total Wins",
                        "BB Period","BB Method","BB Price","BB Dev","Price For Break",
                        "CCI Period","CCI Price","CCI Level");
      }
   }

//+------------------------------------------------------------------+

   return(0);
}

//+------------------------------------------------------------------+
//| Deinitialization function                                        |
//+------------------------------------------------------------------+

int deinit()
{
   if(ShowComments)
   {
      if(CommentsMode == 0) DeleteLabels();
      if(CommentsMode == 1) Comment("");
   }
   if(DrawArrows)   DeleteArrows();
   if(DrawSL_TP)    DeleteSLTP();
   if(WriteCSVFile) { if(file) FileClose(file); }

   return(0);
}

//+------------------------------------------------------------------+
//| Main calculation                                                 |
//+------------------------------------------------------------------+

int start()
{

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+ Avoid refreshing

   if(RunOnlyOnce) { static int CanRun = 0; if(CanRun == 0) CanRun = 1; else return(-1); }

//+------------------------------------------------------------------+ Setting bars

   int counted_bars=IndicatorCounted();
   if (counted_bars<0) return(-1);
   if (counted_bars>0) counted_bars--;
   int i,limit = MathMin(Bars-counted_bars,MathMin(Bars-1,MathMin(Bars-MathMax(BandsPeriod,PeriodMax),Bars-MathMax(CCIPeriod,CCIPeriodMax))));

//+------------------------------------------------------------------+ Setting buffers and prices

   for(i=limit; i>=0; i--)
   {
      Loss[i]           = 0;
      Win[i]            = 0;
      Balance[i]        = 0;
      SL[i]             = 0;
      TP[i]             = 0;
      RecTP[i]          = 0;
      RecSL[i]          = 0;
      Direction[i]      = 0;
      FirstLossShift[i] = 0;
      FirstWinShift[i]  = 0;

      switch(PriceForBreak)
      {
         case 0: { PriceUp[i] = Close[i];                            PriceDn[i] = Close[i];                          } break;
         case 1: { PriceUp[i] = High[i];                             PriceDn[i] = Low[i];                            } break;
         case 2: { PriceUp[i] = (High[i] + Low[i])/2;                PriceDn[i] = (High[i] + Low[i])/2;              } break;
         case 3: { PriceUp[i] = (High[i] + Low[i] + Close[i])/3;     PriceDn[i] = (High[i] + Low[i] + Close[i])/3;   } break;
         case 4: { PriceUp[i] = (High[i] + Low[i] + 2*Close[i])/4;   PriceDn[i] = (High[i] + Low[i] + 2*Close[i])/4; } break;
      }
   }

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+ Optimization of parameters

   string MessageAlert1 = "File created: '"+fileName+"', please check in your MQL4/Files folder.";
   string MessageAlert2 = "Please remove or rename '"+fileName+"' or it will be rewritten on next run.";

   int    Per,bestPer       = 0;
   int    Pri,bestPri       = 0;
   int    Met,bestMet       = 0;
   double Dev,bestDev       = 0;
   int    CCIpe,bestCCIpe   = 0;
   int    CCIpr,bestCCIpr   = 0;
   int    CCIlev,bestCCIlev = 0;
   double Percentage=0,MaxPercentage=0;

//+------------------------------------------------------------------+ Finding best parameters

   if(Optimize)
   {
      for(Per=PeriodMin; Per<=PeriodMax; Per+=PeriodStep)
      {
         for(Pri=PriceMin; Pri<=PriceMax; Pri+=1)
         {
            for(Met=MethodMin; Met<=MethodMax; Met+=1)
            {
               for(Dev=DeviationMin; Dev<=DeviationMax; Dev+=DeviationStep)
               {
                  for(CCIpe=CCIPeriodMin; CCIpe<=CCIPeriodMax; CCIpe+=CCIPeriodStep)
                  {
                     for(CCIpr=CCIPriceMin; CCIpr<=CCIPriceMax; CCIpr+=1)
                     {
                        for(CCIlev=CCILevelMin; CCIlev<=CCILevelMax; CCIlev+=CCILevelStep)
                        {
                           Percentage = Optimization(limit,BarsToCount,Per,Pri,Met,Dev,CCIpe,CCIpr,CCIlev,PriceUp,PriceDn);
                           if(Percentage > MaxPercentage)
                           {
                              bestPer    = Per;
                              bestPri    = Pri;
                              bestMet    = Met;
                              bestDev    = Dev;
                              bestCCIpe  = CCIpe;
                              bestCCIpr  = CCIpr;
                              bestCCIlev = CCIlev;
                              MaxPercentage = Percentage;
                           }
                        }
                     }
                  }
               }
            }
         }
      }

//+------------------------------------------------------------------+ Closing CSV file writing

      if(WriteCSVFile)
      {
         if(file)
         {
            Alert(MessageAlert2);
            Alert(MessageAlert1);
            FileClose(file);
         }
      }

//+------------------------------------------------------------------+ Setting best parameters as used values

      BandPeriod    = bestPer;    // Best period
      BandPrice     = bestPri;    // Best price
      BandMethod    = bestMet;    // Best method
      BandDeviation = bestDev;    // Best deviation
      CCIperiod     = bestCCIpe;  // Best CCI period
      CCIprice      = bestCCIpr;  // Best CCI price
      CCILevel      = bestCCIlev; // Best CCI level

      if(SoundAtTheEnd) PlaySound(Sound);
   }

   else                          // If Optimize is set to False, the external default parameters will be used
   {
      BandPeriod    = BandsPeriod;
      BandPrice     = BandsPrice;
      BandMethod    = BandsMethod;
      BandDeviation = Deviation;
      CCIperiod     = CCIPeriod;
      CCIprice      = CCIPrice;
      CCILevel      = CCIOBOSLevel;
   }

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+ Final calculation either for default or best parameters

   int           limitBars;
   double        deviation;
   double        TV = MarketInfo(NULL,MODE_TICKVALUE);
   static double TotalWin        = 0;
   static double TotalTrades     = 0;
   static double TotalBalance    = 0;
   static double PercentageOfWin = 0;

   if(!UseAllBars && BarsToCount < limit) limitBars = BarsToCount;
   else                                   limitBars = limit;

//+------------------------------------------------------------------+ Setting indicators buffers

   for(i=limit; i>=0; i--)
   {
      MovingBuffer[i] = iMA    (NULL,0,BandPeriod,0,BandMethod,BandPrice,i);
      deviation       = iStdDev(NULL,0,BandPeriod,0,MODE_SMA,BandPrice,i);
      UpperBuffer[i]  = MovingBuffer[i] + deviation * BandDeviation;
      LowerBuffer[i]  = MovingBuffer[i] - deviation * BandDeviation;
      Osc[i]          = iCCI(NULL,0,CCIperiod,CCIprice,i);

//+------------------------------------------------------------------+ Finding signals

      if(i <= limitBars)
      {
         bool dontTrade = PriceUp[i] > UpperBuffer[i] && PriceDn[i] < LowerBuffer[i];

         if     (!dontTrade && Osc[i] >  CCILevel && PriceUp[i] > UpperBuffer[i])    // Sell signals
         {
            SL[i] = iOpen(NULL,0,i-1) + (StopLoss   / MathPow(10,Digits));
            TP[i] = iOpen(NULL,0,i-1) - (TakeProfit / MathPow(10,Digits));
            Direction[i] = -1;
         }

         else if(!dontTrade && Osc[i] < -CCILevel && PriceDn[i] < LowerBuffer[i])    // Buy signals
         {
            SL[i] = iOpen(NULL,0,i-1) - (StopLoss   / MathPow(10,Digits));
            TP[i] = iOpen(NULL,0,i-1) + (TakeProfit / MathPow(10,Digits));
            Direction[i] =  1;
         }

//+------------------------------------------------------------------+ Finding if trade is a winner or loser

         if     (Direction[i] ==  1)
         {
            FirstLossShift[i] = CheckFirstLoserOrWinner(i,True, True, SL[i],TP[i]);
            FirstWinShift[i]  = CheckFirstLoserOrWinner(i,True, False,SL[i],TP[i]);
         }

         else if(Direction[i] == -1)
         {
            FirstLossShift[i] = CheckFirstLoserOrWinner(i,False,True, SL[i],TP[i]);
            FirstWinShift[i]  = CheckFirstLoserOrWinner(i,False,False,SL[i],TP[i]);
         }

         if     (FirstWinShift[i] > FirstLossShift[i])
         {
            Win[i] = 1; Loss[i] = 0; Balance[i] =  TakeProfit*Lots*TV;
         }

         else if(FirstWinShift[i] < FirstLossShift[i])
         {
            Win[i] = 0; Loss[i] = 1; Balance[i] = -StopLoss  *Lots*TV;
         }

//+------------------------------------------------------------------+ Deleting trades when a trade is already open

         if(!AllowMultipleOpenTrades)
         {
            if(TP[i] > 0) RecTP[i] = TP[i]; else if (RecTP[i+1] > 0) RecTP[i] = RecTP[i+1];
            if(SL[i] > 0) RecSL[i] = SL[i]; else if (RecSL[i+1] > 0) RecSL[i] = RecSL[i+1];
   
            if((High[i] < RecTP[i+1] && Low[i] > RecSL[i+1]) || (High[i] < RecSL[i+1] && Low[i] > RecTP[i+1]))
            {
               RecTP[i] = RecTP[i+1];
               RecSL[i] = RecSL[i+1];
            }

            if(TP[i] != RecTP[i] || SL[i] != RecSL[i]) 
            {
               Win[i] = 0; Loss[i] = 0; Balance[i] = 0;
            }
         }

//+------------------------------------------------------------------+ Totals

         TotalWin     += Win[i];
         TotalTrades  += (Win[i] + Loss[i]);
         TotalBalance += Balance[i];

         if(TotalTrades == 0) PercentageOfWin = 0;
         else                 PercentageOfWin = (TotalWin * 100) / TotalTrades;

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+ Objects management

         if(DrawArrows)  ManageArrow(i);
         if(DrawSL_TP)   ManageSLTP(i);
      }
   }

//+------------------------------------------------------------------+ Labels

   if(ShowComments)
   {
      if(OptimizeWhat == 0) string Line1 = "BB giving the best Win Rate when "+priceForBreak+" breaks and entry on next bar:";
      if(OptimizeWhat == 1)        Line1 = "BB giving the best Balance when " +priceForBreak+" breaks and entry on next bar:";
      string Line2 = "Period: "+BandPeriod+" / Method: "+BandMethod+" / Price: "+BandPrice+" / Dev: "+BandDeviation;
      string Line3 = "CCI Period: "+CCIperiod+" / CCI Price: "+CCIprice+" / CCI Level: "+CCILevel;
      string Line4 = "Trades: "+TotalTrades+" / Wins: "+TotalWin+" / Win%: "+DoubleToString(PercentageOfWin,2)+"% / Balance: "+TotalBalance;

      if(CommentsMode == 0)
      {
         color couleur = White;
         int   corner  = 0;
         int   x = 10;
         int   y = 40;

         DrawLabels(1,corner,x,y,   Line1,couleur);
         DrawLabels(2,corner,x,y+10,Line2,couleur);
         DrawLabels(3,corner,x,y+20,Line3,couleur);
         DrawLabels(4,corner,x,y+30,Line4,couleur);
      }

      if(CommentsMode == 1) Comment(Line1,"\n",Line2,"\n",Line3,"\n",Line4);
   }

//+------------------------------------------------------------------+ Writing CSV file when no optimization

   if(!Optimize && WriteCSVFile && !WriteOnlyOnOptimization)
   {
      string bandMethod,bandPrice,cciPrice;
      
      switch(BandMethod)
      {
         case 0: bandMethod = "Simple";          break;
         case 1: bandMethod = "Exponential";     break;
         case 2: bandMethod = "Smoothed";        break;
         case 3: bandMethod = "Linear Weigthed"; break;
      }

      switch(BandPrice)
      {
         case 0: bandPrice = "Close";           break;
         case 1: bandPrice = "Open";            break;
         case 2: bandPrice = "High";            break;
         case 3: bandPrice = "Low";             break;
         case 4: bandPrice = "Median";          break;
         case 5: bandPrice = "Typical";         break;
         case 6: bandPrice = "Weigthed Close";  break;
      }

      switch(CCIprice)
      {
         case 0: cciPrice = "Close";           break;
         case 1: cciPrice = "Open";            break;
         case 2: cciPrice = "High";            break;
         case 3: cciPrice = "Low";             break;
         case 4: cciPrice = "Median";          break;
         case 5: cciPrice = "Typical";         break;
         case 6: cciPrice = "Weigthed Close";  break;
      }

      double Return = DoubleToString(PercentageOfWin,2);
      FileWrite(file,TotalBalance,Return,TotalTrades,TotalWin,
                     BandPeriod,bandMethod,bandPrice,BandDeviation,priceForBreak,
                     CCIperiod,cciPrice,CCILevel);
      if(file)
      {
         Alert(MessageAlert2);
         Alert(MessageAlert1);
         FileClose(file);
      }
   }

//+------------------------------------------------------------------+ Indicator Name

   IndicatorShortName("Bollinger Bands ("+BandPeriod+","+BandPrice+","+BandMethod+","+BandDeviation+
                      ") + CCI ("+CCIperiod+","+CCIprice+","+CCILevel+")");

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+

   return(0);
}

//+---------------------------------------------------------------------------------------+
//| Functions                                                                             |
//+---------------------------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Optimization                                                     |
//+------------------------------------------------------------------+

double Optimization(int limit, int barsLimit, int period, int price, int method, double deviation, int cciPeriod, int cciPrice, int cciLevel,
                    double& priceUp[], double& priceDn[])
{
   int    i,direction = 0, firstLossShift = 0, firstWinShift = 0;
   double sl          = 0, tp             = 0;
   double loss        = 0, win            = 0, balance       = 0;
   double totalWin    = 0, totalTrades    = 0, totalBalance  = 0;
   double result      = 0;

   double Ma,Dev,Up,Dn,cci;
   double recTP[],recSL[];

   double TV = MarketInfo(NULL,MODE_TICKVALUE);

//+------------------------------------------------------------------+ Setting indicators

   for(i=limit; i>=0; i--)
   {
      Ma  = iMA    (NULL,0,period,0,method,price,i);
      Dev = iStdDev(NULL,0,period,0,MODE_SMA,price,i);
      Up  = Ma + Dev * deviation;
      Dn  = Ma - Dev * deviation;
      cci = iCCI(NULL,0,cciPeriod,cciPrice,i);
      
      recTP[i] = 0;
      recSL[i] = 0;

      if(UseAllBars || barsLimit > limit) barsLimit = limit;

//+------------------------------------------------------------------+ Finding signals

      if(i <= barsLimit)
      {
         bool doNotTrade = PriceUp[i] > UpperBuffer[i] && PriceDn[i] < LowerBuffer[i];
         
         if     (!doNotTrade && cci >  cciLevel && priceUp[i] > Up)
         {
            sl = iOpen(NULL,0,i-1) + (StopLoss   / MathPow(10,Digits));
            tp = iOpen(NULL,0,i-1) - (TakeProfit / MathPow(10,Digits));
            direction = -1;
         }
         
         else if(!doNotTrade && cci < -cciLevel && priceUp[i] < Dn) 
         {
            sl = iOpen(NULL,0,i-1) - (StopLoss   / MathPow(10,Digits));
            tp = iOpen(NULL,0,i-1) + (TakeProfit / MathPow(10,Digits));
            direction =  1;
         }

//+------------------------------------------------------------------+ Finding if trade is a winner or loser

         if     (direction ==  1)
         {
            firstLossShift = CheckFirstLoserOrWinner(i,True, True, sl,tp);
            firstWinShift  = CheckFirstLoserOrWinner(i,True, False,sl,tp);
         }

         else if(direction == -1)
         {
            firstLossShift = CheckFirstLoserOrWinner(i,False,True, sl,tp);
            firstWinShift  = CheckFirstLoserOrWinner(i,False,False,sl,tp);
         }

         if     (firstWinShift > firstLossShift)
         {
            win = 1; loss = 0; balance =  TakeProfit*Lots*TV;
         }

         else if(firstWinShift < firstLossShift)
         {
            win = 0; loss = 1; balance = -StopLoss  *Lots*TV;
         }

//+------------------------------------------------------------------+ Deleting trades when a trade is already open

         if(!AllowMultipleOpenTrades)
         {
            if(tp > 0) recTP[i] = tp; else if (recTP[i+1] > 0) recTP[i] = recTP[i+1];
            if(sl > 0) recSL[i] = sl; else if (recSL[i+1] > 0) recSL[i] = recSL[i+1];
   
            if((High[i] < recTP[i+1] && Low[i] > recSL[i+1]) || (High[i] < recSL[i+1] && Low[i] > recTP[i+1]))
            {
               recTP[i] = recTP[i+1];
               recSL[i] = recSL[i+1];
            }

            if(tp != recTP[i] || sl != recSL[i]) 
            {
               win = 0; loss = 0; balance = 0;
            }
         }

//+------------------------------------------------------------------+ Totals

         totalWin     += win;
         totalTrades  += (win + loss);
         totalBalance += balance;

         if     (OptimizeWhat == 0)
         {
            if(totalTrades == 0) result = 0;
            else                 result = (totalWin * 100) / totalTrades;
         }

         else if(OptimizeWhat == 1)
         {
            result = totalBalance;
         }
      }
   }

//+------------------------------------------------------------------+ Writing CSV file

   if(WriteCSVFile)
   {
      string bandMethod,bandPrice,cciprice;
      
      switch(method)
      {
         case 0: bandMethod = "Simple";          break;
         case 1: bandMethod = "Exponential";     break;
         case 2: bandMethod = "Smoothed";        break;
         case 3: bandMethod = "Linear Weigthed"; break;
      }

      switch(price)
      {
         case 0: bandPrice = "Close";           break;
         case 1: bandPrice = "Open";            break;
         case 2: bandPrice = "High";            break;
         case 3: bandPrice = "Low";             break;
         case 4: bandPrice = "Median";          break;
         case 5: bandPrice = "Typical";         break;
         case 6: bandPrice = "Weigthed Close";  break;
      }

      switch(cciPrice)
      {
         case 0: cciprice = "Close";           break;
         case 1: cciprice = "Open";            break;
         case 2: cciprice = "High";            break;
         case 3: cciprice = "Low";             break;
         case 4: cciprice = "Median";          break;
         case 5: cciprice = "Typical";         break;
         case 6: cciprice = "Weigthed Close";  break;
      }

      FileWrite(file,balance,DoubleToString((double)result,2),totalTrades,totalWin,
                     period,bandMethod,bandPrice,deviation,priceForBreak,
                     cciPeriod,cciprice,cciLevel);
   }

//+------------------------------------------------------------------+ Freeing used arrays

   ArrayFree(recTP);
   ArrayFree(recSL);

   return(result);
}

//+---------------------------------------------------------------------------------------+
//| Other functions                                                                       |
//+---------------------------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Text labels                                                      |
//+------------------------------------------------------------------+

void DrawLabels(int instance, int corner, int x, int y, string text, color couleur)
{
   string name = "Percentage_"+instance;
   ObjectCreate (name,OBJ_LABEL,0,0,0);
   ObjectSetText(name,text,7,"Verdana",couleur);
   ObjectSet(name,OBJPROP_CORNER,corner);
   ObjectSet(name,OBJPROP_XDISTANCE,x);
   ObjectSet(name,OBJPROP_YDISTANCE,y);
}

//+------------------------------------------------------------------+

void DeleteLabels()
{
   for(int i=ObjectsTotal()-1; i>=0; i--)
   {
       string name  = "Percentage_";
       string label = ObjectName(i);
       if(StringSubstr(label,0,StringLen(name)) != name) continue;
       ObjectDelete(label);
   }
}

//+------------------------------------------------------------------+
//| Arrows                                                           |
//+------------------------------------------------------------------+

void ManageArrow(int i)
{
   DeleteArrow(Time[i]);

   if(Direction[i] ==  1 && Win[i]  == 1)
   DrawArrow(i,Lime,233,1,False);
   if(Direction[i] ==  1 && Loss[i] == 1)
   DrawArrow(i,Red, 233,1,False);

   if(Direction[i] == -1 && Win[i]  == 1)
   DrawArrow(i,Lime,234,1,True);
   if(Direction[i] == -1 && Loss[i] == 1)
   DrawArrow(i,Red, 234,1,True);
}

//-------------------------------------------------------------------+

void DrawArrow(int i, color theColor, int theCode, int theWidth, bool up)
{
   string name = "Arrows_"+Time[i];
   double gap  = (3.0*iATR(NULL,0,20,i)/4.0) * 0.8;
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i]+ShowArrowsOnEntryCandle*Period()*60,0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         ObjectSet(name,OBJPROP_WIDTH,theWidth);
         ObjectSet(name,OBJPROP_TIME1,Time[i]+ShowArrowsOnEntryCandle*Period()*60);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] -gap);
}

//-------------------------------------------------------------------+

void DeleteArrows()
{
   string lookFor       = "Arrows_";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//-------------------------------------------------------------------+

void DeleteArrow(datetime time)
{
   string lookFor = "Arrows_"+time; ObjectDelete(lookFor);
}

//+------------------------------------------------------------------+
//| SL and TP                                                        |
//+------------------------------------------------------------------+

void ManageSLTP(int i)
{
   DeleteSlTp(Time[i]);

   if(Win[i] != 0 || Loss[i] != 0)
   {
      DrawSLTP(i,Blue,4,1,TP,"TP_");
      DrawSLTP(i,Red, 4,1,SL,"SL_");
   }
}

//-------------------------------------------------------------------+

void DrawSLTP(int i, color theColor, int theCode, int theWidth, double& price[], string whichOne)
{
   string name = whichOne+Time[i];

      ObjectCreate(name,OBJ_ARROW,0,Time[i]+ShowSL_TPOnEntryCandle*Period()*60,price[i]);
      ObjectSet(name,OBJPROP_ARROWCODE,theCode);
      ObjectSet(name,OBJPROP_COLOR,theColor);
      ObjectSet(name,OBJPROP_WIDTH,theWidth);
      ObjectSet(name,OBJPROP_PRICE1,price[i]);
      ObjectSet(name,OBJPROP_TIME1,Time[i]+ShowSL_TPOnEntryCandle*Period()*60);
}

//-------------------------------------------------------------------+

void DeleteSLTP()
{
   string lookFor1       = "SL_";
   string lookFor2       = "TP_";
   int    lookForLength1 = StringLen(lookFor1);
   int    lookForLength2 = StringLen(lookFor2);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength1) == lookFor1) ObjectDelete(objectName);
         if (StringSubstr(objectName,0,lookForLength2) == lookFor2) ObjectDelete(objectName);
   }
}

//+------------------------------------------------------------------+

void DeleteSlTp(datetime time)
{
   string lookFor1 = "SL_"+time; ObjectDelete(lookFor1);
   string lookFor2 = "TP_"+time; ObjectDelete(lookFor2);
}

//+------------------------------------------------------------------+
//| TimeFrameToString                                                |
//+------------------------------------------------------------------+

string TimeFrameToString(int tf)
{
   string tfs;
   switch(tf)
   {
      case PERIOD_M1:     tfs = "M1";      break;
      case PERIOD_M5:     tfs = "M5";      break;
      case PERIOD_M15:    tfs = "M15";     break;
      case PERIOD_M30:    tfs = "M30";     break;
      case PERIOD_H1:     tfs = "H1";      break;
      case PERIOD_H4:     tfs = "H4";      break;
      case PERIOD_D1:     tfs = "D1";      break;
      case PERIOD_W1:     tfs = "W1";      break;
      case PERIOD_MN1:    tfs = "MN1";     break;
      default: tfs = "M" + DoubleToStr(tf,0);
   }
   return(tfs);
}

//+------------------------------------------------------------------+
//| CheckFirstLoserOrWinner                                          |
//+------------------------------------------------------------------+

int CheckFirstLoserOrWinner(int i, bool up, bool loss, double Sl, double Tp)
{
   int k,pre=0,result=0;

   for(k=i-1; k>=0; k--)
   {
      if(up)
      {
         if(loss) { if(iHigh(NULL,0,k) >= Sl                                 && iLow(NULL,0,k) <= Sl) { pre = k; if(pre > result) result = pre; } }
         else     { if(iHigh(NULL,0,k) + (Spread / MathPow(10,Digits)) >= Tp && iLow(NULL,0,k) <= Tp) { pre = k; if(pre > result) result = pre; } }
      }

      else
      {
         if(loss) { if(iHigh(NULL,0,k) + (Spread / MathPow(10,Digits)) >= Sl && iLow(NULL,0,k) <= Sl) { pre = k; if(pre > result) result = pre; } }
         else     { if(iHigh(NULL,0,k) >= Tp && iLow(NULL,0,k) + (Spread / MathPow(10,Digits)) <= Tp) { pre = k; if(pre > result) result = pre; } }
      }
   }

   return(result);
}

//+------------------------------------------------------------------+
//| The End                                                          |
//+------------------------------------------------------------------+